"""Examples of using the UK Grid Carbon Intensity API client."""

import asyncio
from datetime import datetime, date, timedelta

from uk_grid_intensity import CarbonIntensityClient
from uk_grid_intensity.constants import REGION_NAMES


async def basic_examples():
    """Basic usage examples."""
    print("=== Basic Examples ===")

    # Create client (can be used as context manager)
    async with CarbonIntensityClient() as client:

        # Get current intensity
        print("\n1. Current Carbon Intensity:")
        current = await client.aget_current_intensity()
        for data in current:
            print(f"  Period: {data.from_} to {data.to}")
            print(f"  Forecast: {data.intensity.forecast} gCO2/kWh")
            print(f"  Index: {data.intensity.index}")

        # Get intensity for specific date
        print("\n2. Carbon Intensity for Today:")
        today_data = await client.aget_intensity_today()
        print(f"  Found {len(today_data)} half-hour periods")

        # Get generation mix
        print("\n3. Current Generation Mix:")
        generation = await client.aget_current_generation_mix()
        print(f"  Period: {generation.from_} to {generation.to}")
        for fuel in generation.generationmix:
            print(f"    {fuel.fuel}: {fuel.perc}%")

        # Get regional data
        print("\n4. Regional Data for London:")
        london_data = await client.aget_intensity_by_region_id(
            13
        )  # London is region 13
        for region in london_data:
            print(f"  Region: {region.shortname}")
            for data_point in region.data:
                print(f"    {data_point.from_} to {data_point.to}")
                print(
                    f"    Intensity: {data_point.intensity.forecast} gCO2/kWh ({data_point.intensity.index})"
                )


async def advanced_examples():
    """Advanced usage examples."""
    print("\n=== Advanced Examples ===")

    client = CarbonIntensityClient()

    try:
        # Get statistics for the past week
        print("\n1. Weekly Statistics:")
        end_date = datetime.now()
        start_date = end_date - timedelta(days=7)

        stats = await client.aget_intensity_statistics(start_date, end_date)
        for data in stats:
            print(f"  Period: {data.from_} to {data.to}")
            print(f"    Average: {data.intensity.average} gCO2/kWh")
            print(f"    Min: {data.intensity.min} gCO2/kWh")
            print(f"    Max: {data.intensity.max} gCO2/kWh")
            print(f"    Index: {data.intensity.index}")

        # Get intensity factors
        print("\n2. Carbon Intensity Factors:")
        factors = await client.aget_intensity_factors()
        for factor_data in factors:
            factor_dict = factor_data.model_dump()
            for fuel_type, factor_value in factor_dict.items():
                if factor_value is not None:
                    print(f"  {fuel_type}: {factor_value} gCO2/kWh")

        # Get forecast for next 24 hours
        print("\n3. 24-Hour Forecast:")
        forecast = await client.aget_intensity_forward_24h(datetime.now())
        print(f"  Found {len(forecast)} forecast periods")

        # Find the cleanest time in the next 24 hours
        if forecast:
            cleanest = min(forecast, key=lambda x: x.intensity.forecast or float("inf"))
            print(f"  Cleanest period: {cleanest.from_} to {cleanest.to}")
            print(
                f"  Intensity: {cleanest.intensity.forecast} gCO2/kWh ({cleanest.intensity.index})"
            )

        # Get data for a specific postcode
        print("\n4. Data for Postcode SW1A 1AA:")
        postcode_data = await client.aget_intensity_by_postcode("SW1A1AA")
        for region in postcode_data:
            print(f"  Region: {region.shortname} (ID: {region.regionid})")
            print(f"  DNO: {region.dnoregion}")

    finally:
        await client.aclose()


def synchronous_examples():
    """Synchronous usage examples."""
    print("\n=== Synchronous Examples ===")

    # Using context manager
    with CarbonIntensityClient() as client:

        # Get current intensity
        print("\n1. Current Intensity (Sync):")
        current = client.get_current_intensity()
        for data in current:
            print(f"  {data.intensity.forecast} gCO2/kWh ({data.intensity.index})")

        # Get intensity by date and period
        print("\n2. Specific Date and Period:")
        try:
            specific = client.get_intensity_by_date_and_period(
                date.today(), 25
            )  # Period 25 (12:00-12:30)
            for data in specific:
                print(f"  Period 25: {data.intensity.forecast} gCO2/kWh")
        except Exception as e:
            print(f"  Error: {e}")

        # Get all regions
        print("\n3. All Regions:")
        regional = client.get_current_regional_intensity()
        for time_data in regional:
            print(f"  Time: {time_data.from_} to {time_data.to}")
            for region in time_data.regions:
                region_name = REGION_NAMES.get(
                    region.regionid, f"Region {region.regionid}"
                )
                print(f"    {region_name}: {region.intensity.forecast} gCO2/kWh")


def error_handling_examples():
    """Error handling examples."""
    print("\n=== Error Handling Examples ===")

    from uk_grid_intensity import CarbonIntensityAPIError

    client = CarbonIntensityClient()

    try:
        # Invalid region ID
        print("\n1. Testing Invalid Region ID:")
        try:
            client.get_intensity_by_region_id(99)  # Invalid region ID
        except ValueError as e:
            print(f"  Validation Error: {e}")

        # Invalid date format (this would be caught by datetime parsing)
        print("\n2. Testing Invalid Date:")
        try:
            client.get_intensity_by_date("invalid-date")
        except Exception as e:
            print(f"  Date Error: {e}")

        # API error simulation (using non-existent endpoint)
        print("\n3. Testing API Error:")
        try:
            # This would trigger an HTTP error if the endpoint doesn't exist
            response = client._client.get("/nonexistent")
            client._handle_response(response)
        except CarbonIntensityAPIError as e:
            print(f"  API Error: {e.message}")
            print(f"  Status Code: {e.status_code}")

    finally:
        client.close()


def data_analysis_example():
    """Example of analyzing carbon intensity data."""
    print("\n=== Data Analysis Example ===")

    with CarbonIntensityClient() as client:

        # Get today's data
        today_data = client.get_intensity_today()

        if today_data:
            intensities = [
                d.intensity.forecast for d in today_data if d.intensity.forecast
            ]

            if intensities:
                avg_intensity = sum(intensities) / len(intensities)
                min_intensity = min(intensities)
                max_intensity = max(intensities)

                print("\nToday's Carbon Intensity Analysis:")
                print(f"  Periods analyzed: {len(intensities)}")
                print(f"  Average: {avg_intensity:.1f} gCO2/kWh")
                print(f"  Minimum: {min_intensity} gCO2/kWh")
                print(f"  Maximum: {max_intensity} gCO2/kWh")
                print(f"  Range: {max_intensity - min_intensity} gCO2/kWh")

                # Find cleanest and dirtiest periods
                cleanest = min(
                    today_data, key=lambda x: x.intensity.forecast or float("inf")
                )
                dirtiest = max(today_data, key=lambda x: x.intensity.forecast or 0)

                print(
                    f"\n  Cleanest period: {cleanest.from_.strftime('%H:%M')} - {cleanest.to.strftime('%H:%M')}"
                )
                print(f"    Intensity: {cleanest.intensity.forecast} gCO2/kWh")
                print(
                    f"  Dirtiest period: {dirtiest.from_.strftime('%H:%M')} - {dirtiest.to.strftime('%H:%M')}"
                )
                print(f"    Intensity: {dirtiest.intensity.forecast} gCO2/kWh")


async def main():
    """Run all examples."""
    print("UK Grid Carbon Intensity API Client Examples")
    print("=" * 50)

    await basic_examples()
    await advanced_examples()
    synchronous_examples()
    error_handling_examples()
    data_analysis_example()

    print("\n" + "=" * 50)
    print("Examples completed!")


if __name__ == "__main__":
    asyncio.run(main())



"""UK Grid Carbon Intensity API Client.

A Python client for accessing the UK National Grid Carbon Intensity API.
Provides comprehensive access to carbon intensity data, generation mix information,
regional data, and statistics with full type safety using Pydantic models.
"""

from .client import CarbonIntensityAPIError, CarbonIntensityClient
from .schemas import (  # Response types; Data models; Nested models; Enums
    ErrorDetails,
    ErrorResponse,
    FactorsData,
    FactorsResponse,
    FuelMix,
    FuelType,
    GenerationData,
    GenerationResponse,
    IntensityData,
    IntensityIndex,
    IntensityResponse,
    IntensityValue,
    RegionalDataPoint,
    RegionalFromTo,
    RegionalId,
    RegionalIntensityData,
    RegionalResponse,
    StatisticsData,
    StatisticsIntensity,
    StatisticsResponse,
)

__version__ = "0.1.0"

__all__ = [
    # Client
    "CarbonIntensityClient",
    "CarbonIntensityAPIError",
    # Response types
    "IntensityResponse",
    "GenerationResponse",
    "RegionalResponse",
    "StatisticsResponse",
    "FactorsResponse",
    "ErrorResponse",
    # Data models
    "IntensityData",
    "GenerationData",
    "RegionalId",
    "RegionalFromTo",
    "StatisticsData",
    "FactorsData",
    # Nested models
    "IntensityValue",
    "StatisticsIntensity",
    "FuelMix",
    "RegionalIntensityData",
    "RegionalDataPoint",
    "ErrorDetails",
    # Enums
    "IntensityIndex",
    "FuelType",
]



"""Pydantic schemas for the UK Carbon Intensity API.

This module defines all the data models used by the Carbon Intensity API,
providing type safety and validation for API responses.
"""

from datetime import datetime
from enum import Enum
from typing import List, Optional, Union

from pydantic import BaseModel, ConfigDict, Field, field_validator, model_validator


class IntensityIndex(str, Enum):
    """Carbon intensity index levels."""

    VERY_LOW = "very low"
    LOW = "low"
    MODERATE = "moderate"
    HIGH = "high"
    VERY_HIGH = "very high"


class FuelType(str, Enum):
    """Fuel types used in generation mix."""

    GAS = "gas"
    COAL = "coal"
    BIOMASS = "biomass"
    NUCLEAR = "nuclear"
    HYDRO = "hydro"
    IMPORTS = "imports"
    OTHER = "other"
    WIND = "wind"
    SOLAR = "solar"
    STORAGE = "storage"  # Pumped storage


class IntensityValue(BaseModel):
    """Carbon intensity values with forecast, actual, and index."""

    forecast: Optional[int] = Field(
        None, description="Forecast carbon intensity in gCO2/kWh"
    )
    actual: Optional[int] = Field(
        None, description="Actual carbon intensity in gCO2/kWh"
    )
    index: IntensityIndex = Field(..., description="Intensity index level")


class StatisticsIntensity(BaseModel):
    """Carbon intensity statistics with max, average, min, and index."""

    max: int = Field(..., description="Maximum carbon intensity in gCO2/kWh")
    average: int = Field(..., description="Average carbon intensity in gCO2/kWh")
    min: int = Field(..., description="Minimum carbon intensity in gCO2/kWh")
    index: IntensityIndex = Field(..., description="Intensity index level")


class FuelMix(BaseModel):
    """Individual fuel type in generation mix."""

    fuel: Union[FuelType, str] = Field(..., description="Fuel type")
    perc: float = Field(..., description="Percentage of total generation", ge=0, le=100)


class IntensityData(BaseModel):
    """Core intensity data with time period and intensity values."""

    from_: datetime = Field(..., alias="from", description="Start datetime in UTC")
    to: datetime = Field(..., description="End datetime in UTC")
    intensity: IntensityValue = Field(..., description="Carbon intensity data")


class GenerationData(BaseModel):
    """Generation mix data with time period."""

    from_: datetime = Field(..., alias="from", description="Start datetime in UTC")
    to: datetime = Field(..., description="End datetime in UTC")
    generationmix: List[FuelMix] = Field(..., description="Generation mix by fuel type")


class StatisticsData(BaseModel):
    """Statistics data with time period and intensity statistics."""

    from_: datetime = Field(..., alias="from", description="Start datetime in UTC")
    to: datetime = Field(..., description="End datetime in UTC")
    intensity: StatisticsIntensity = Field(
        ..., description="Carbon intensity statistics"
    )


class RegionalIntensityData(BaseModel):
    """Regional intensity data with time period, intensity, and generation mix."""

    from_: datetime = Field(..., alias="from", description="Start datetime in UTC")
    to: datetime = Field(..., description="End datetime in UTC")
    intensity: IntensityValue = Field(..., description="Carbon intensity data")
    generationmix: Optional[List[FuelMix]] = Field(
        None, description="Generation mix by fuel type"
    )


class RegionalDataPoint(BaseModel):
    """Individual region data point."""

    regionid: int = Field(..., description="Region ID (1-17)", ge=1, le=17)
    dnoregion: str = Field(..., description="Distribution Network Operator region name")
    shortname: str = Field(..., description="Short region name")
    intensity: IntensityValue = Field(..., description="Carbon intensity data")
    generationmix: Optional[List[FuelMix]] = Field(
        None, description="Generation mix by fuel type"
    )


class RegionalFromTo(BaseModel):
    """Regional data for a specific time period."""

    from_: datetime = Field(..., alias="from", description="Start datetime in UTC")
    to: datetime = Field(..., description="End datetime in UTC")
    regions: List[RegionalDataPoint] = Field(..., description="Regional data points")


class RegionalId(BaseModel):
    """Regional data for a specific region with historical data."""

    regionid: int = Field(..., description="Region ID (1-17)", ge=1, le=17)
    dnoregion: str = Field(..., description="Distribution Network Operator region name")
    shortname: str = Field(..., description="Short region name")
    postcode: Optional[str] = Field(None, description="Outward postcode")
    data: List[RegionalIntensityData] = Field(
        ..., description="Time series intensity data"
    )


class FactorsData(BaseModel):
    """Carbon intensity factors for different fuel types."""

    biomass: Optional[int] = Field(None, alias="Biomass")
    coal: Optional[int] = Field(None, alias="Coal")
    dutch_imports: Optional[int] = Field(None, alias="Dutch Imports")
    french_imports: Optional[int] = Field(None, alias="French Imports")
    gas_combined_cycle: Optional[int] = Field(None, alias="Gas (Combined Cycle)")
    gas_open_cycle: Optional[int] = Field(None, alias="Gas (Open Cycle)")
    hydro: Optional[int] = Field(None, alias="Hydro")
    irish_imports: Optional[int] = Field(None, alias="Irish Imports")
    nuclear: Optional[int] = Field(None, alias="Nuclear")
    oil: Optional[int] = Field(None, alias="Oil")
    other: Optional[int] = Field(None, alias="Other")
    pumped_storage: Optional[int] = Field(None, alias="Pumped Storage")
    solar: Optional[int] = Field(None, alias="Solar")
    wind: Optional[int] = Field(None, alias="Wind")


class ErrorDetails(BaseModel):
    """Error details."""

    code: str = Field(..., description="HTTP error code")
    message: str = Field(..., description="Error message")


class ErrorResponse(BaseModel):
    """API error response."""

    error: ErrorDetails = Field(..., description="Error details")


# Response models
class IntensityResponse(BaseModel):
    """Response model for intensity endpoints."""

    data: List[IntensityData] = Field(..., description="Intensity data list")


class GenerationResponse(BaseModel):
    """Response model for generation endpoints."""

    data: GenerationData = Field(..., description="Generation data")


class StatisticsResponse(BaseModel):
    """Response model for statistics endpoints."""

    data: List[StatisticsData] = Field(..., description="Statistics data list")


class RegionalFromToResponse(BaseModel):
    """Response model for regional endpoints, list based on regions, national or country level."""

    data: List[RegionalFromTo] = Field(
        ..., description="Regional data list with a list of regional data"
    )


class RegionalIdResponse(BaseModel):
    """Response model for regional endpoints, list based on ID."""

    data: List[RegionalId] = Field(
        ..., description="Regional data list with data for each region ID"
    )


class RegionalResponse(BaseModel):
    """Response model for regional endpoints."""

    data: List[Union[RegionalFromTo, RegionalId]] = Field(
        ..., description="Regional data list"
    )


class FactorsResponse(BaseModel):
    """Response model for factors endpoint."""

    data: List[FactorsData] = Field(..., description="Factors data list")


GenerationMix = List[FuelMix]



"""Carbon Intensity API Client.

A comprehensive client for the UK National Grid Carbon Intensity API.
Provides methods to access all API endpoints with proper type safety.
"""

import asyncio
from datetime import date, datetime
from typing import List, Optional, Union
from urllib.parse import urljoin

import httpx

from .schemas import (
    FactorsData,
    FactorsResponse,
    GenerationData,
    GenerationResponse,
    IntensityData,
    IntensityResponse,
    RegionalFromTo,
    RegionalFromToResponse,
    RegionalId,
    RegionalIdResponse,
    RegionalResponse,
    StatisticsData,
    StatisticsResponse,
)

# Constants
_REGION_ID_ERROR = "Region ID must be between 1 and 17"


class CarbonIntensityAPIError(Exception):
    """Exception raised when the Carbon Intensity API returns an error."""

    def __init__(
        self,
        message: str,
        status_code: Optional[int] = None,
        error_code: Optional[str] = None,
    ):
        self.message = message
        self.status_code = status_code
        self.error_code = error_code
        super().__init__(message)


class CarbonIntensityClient:
    """Client for the UK Carbon Intensity API.

    Provides synchronous and asynchronous methods to access carbon intensity data,
    generation mix information, regional data, and statistics.

    Args:
        base_url: Base URL for the API (default: https://api.carbonintensity.org.uk)
        timeout: Request timeout in seconds (default: 30)
        user_agent: User agent string for requests
    """

    def __init__(
        self,
        base_url: str = "https://api.carbonintensity.org.uk",
        timeout: float = 30.0,
        user_agent: Optional[str] = None,
    ):
        self.base_url = base_url.rstrip("/")
        self.timeout = timeout
        self.user_agent = user_agent or "uk-grid-intensity-client/0.1.0"

        # Configure httpx client
        self._client = httpx.Client(
            base_url=self.base_url,
            timeout=self.timeout,
            headers={"User-Agent": self.user_agent},
        )
        self._async_client: Optional[httpx.AsyncClient] = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        await self.aclose()

    def close(self):
        """Close the synchronous client."""
        self._client.close()

    async def aclose(self):
        """Close the asynchronous client."""
        if self._async_client:
            await self._async_client.aclose()

    @property
    def async_client(self) -> httpx.AsyncClient:
        """Get or create the async client."""
        if self._async_client is None:
            self._async_client = httpx.AsyncClient(
                base_url=self.base_url,
                timeout=self.timeout,
                headers={"User-Agent": self.user_agent},
            )
        return self._async_client

    def _handle_response(self, response: httpx.Response) -> dict:
        """Handle API response and check for errors."""
        try:
            response.raise_for_status()
            return response.json()
        except httpx.HTTPStatusError:
            try:
                error_data = response.json()
                if "error" in error_data:
                    error = error_data["error"]
                    raise CarbonIntensityAPIError(
                        error.get("message", "Unknown API error"),
                        response.status_code,
                        error.get("code"),
                    )
            except (ValueError, KeyError):
                pass

            raise CarbonIntensityAPIError(
                f"HTTP {response.status_code}: {response.text}", response.status_code
            )

    def _format_datetime(self, dt: Union[datetime, str]) -> str:
        """Format datetime for API requests."""
        if isinstance(dt, str):
            return dt
        return dt.strftime("%Y-%m-%dT%H:%MZ")

    def _format_date(self, d: Union[date, str]) -> str:
        """Format date for API requests."""
        if isinstance(d, str):
            return d
        return d.strftime("%Y-%m-%d")

    # Intensity endpoints
    def get_current_intensity(self) -> List[IntensityData]:
        """Get carbon intensity data for current half hour."""
        response = self._client.get("/intensity")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    async def aget_current_intensity(self) -> List[IntensityData]:
        """Get carbon intensity data for current half hour (async)."""
        response = await self.async_client.get("/intensity")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    def get_intensity_today(self) -> List[IntensityData]:
        """Get carbon intensity data for today."""
        response = self._client.get("/intensity/date")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    async def aget_intensity_today(self) -> List[IntensityData]:
        """Get carbon intensity data for today (async)."""
        response = await self.async_client.get("/intensity/date")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    def get_intensity_by_date(self, date_: Union[date, str]) -> List[IntensityData]:
        """Get carbon intensity data for specific date."""
        date_str = self._format_date(date_)
        response = self._client.get(f"/intensity/date/{date_str}")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    async def aget_intensity_by_date(
        self, date_: Union[date, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data for specific date (async)."""
        date_str = self._format_date(date_)
        response = await self.async_client.get(f"/intensity/date/{date_str}")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    def get_intensity_by_date_and_period(
        self, date_: Union[date, str], period: int
    ) -> List[IntensityData]:
        """Get carbon intensity data for specific date and settlement period (1-48)."""
        if not 1 <= period <= 48:
            raise ValueError("Period must be between 1 and 48")

        date_str = self._format_date(date_)
        response = self._client.get(f"/intensity/date/{date_str}/{period}")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    async def aget_intensity_by_date_and_period(
        self, date_: Union[date, str], period: int
    ) -> List[IntensityData]:
        """Get carbon intensity data for specific date and settlement period (async)."""
        if not 1 <= period <= 48:
            raise ValueError("Period must be between 1 and 48")

        date_str = self._format_date(date_)
        response = await self.async_client.get(f"/intensity/date/{date_str}/{period}")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    def get_intensity_factors(self) -> List[FactorsData]:
        """Get carbon intensity factors for each fuel type."""
        response = self._client.get("/intensity/factors")
        data = self._handle_response(response)
        return FactorsResponse.model_validate(data).data

    async def aget_intensity_factors(self) -> List[FactorsData]:
        """Get carbon intensity factors for each fuel type (async)."""
        response = await self.async_client.get("/intensity/factors")
        data = self._handle_response(response)
        return FactorsResponse.model_validate(data).data

    def get_intensity_by_datetime(
        self, from_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data for specific datetime."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(f"/intensity/{from_str}")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    async def aget_intensity_by_datetime(
        self, from_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data for specific datetime (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(f"/intensity/{from_str}")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    def get_intensity_forward_24h(
        self, from_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data 24 hours forward from datetime."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(f"/intensity/{from_str}/fw24h")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    async def aget_intensity_forward_24h(
        self, from_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data 24 hours forward from datetime (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(f"/intensity/{from_str}/fw24h")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    def get_intensity_forward_48h(
        self, from_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data 48 hours forward from datetime."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(f"/intensity/{from_str}/fw48h")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    async def aget_intensity_forward_48h(
        self, from_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data 48 hours forward from datetime (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(f"/intensity/{from_str}/fw48h")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    def get_intensity_past_24h(
        self, from_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data 24 hours in the past from datetime."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(f"/intensity/{from_str}/pt24h")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    async def aget_intensity_past_24h(
        self, from_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data 24 hours in the past from datetime (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(f"/intensity/{from_str}/pt24h")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    def get_intensity_between_datetimes(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data between two datetimes (max 14 days)."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = self._client.get(f"/intensity/{from_str}/{to_str}")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    async def aget_intensity_between_datetimes(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str]
    ) -> List[IntensityData]:
        """Get carbon intensity data between two datetimes (async)."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = await self.async_client.get(f"/intensity/{from_str}/{to_str}")
        data = self._handle_response(response)
        return IntensityResponse.model_validate(data).data

    # Statistics endpoints
    def get_intensity_statistics(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str]
    ) -> List[StatisticsData]:
        """Get carbon intensity statistics between datetimes (max 30 days)."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = self._client.get(f"/intensity/stats/{from_str}/{to_str}")
        data = self._handle_response(response)
        return StatisticsResponse.model_validate(data).data

    async def aget_intensity_statistics(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str]
    ) -> List[StatisticsData]:
        """Get carbon intensity statistics between datetimes (async)."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = await self.async_client.get(f"/intensity/stats/{from_str}/{to_str}")
        data = self._handle_response(response)
        return StatisticsResponse.model_validate(data).data

    def get_intensity_statistics_with_blocks(
        self,
        from_dt: Union[datetime, str],
        to_dt: Union[datetime, str],
        block_hours: int,
    ) -> List[StatisticsData]:
        """Get block average carbon intensity statistics (block 1-24 hours)."""
        if not 1 <= block_hours <= 24:
            raise ValueError("Block hours must be between 1 and 24")

        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = self._client.get(
            f"/intensity/stats/{from_str}/{to_str}/{block_hours}"
        )
        data = self._handle_response(response)
        return StatisticsResponse.model_validate(data).data

    async def aget_intensity_statistics_with_blocks(
        self,
        from_dt: Union[datetime, str],
        to_dt: Union[datetime, str],
        block_hours: int,
    ) -> List[StatisticsData]:
        """Get block average carbon intensity statistics (async)."""
        if not 1 <= block_hours <= 24:
            raise ValueError("Block hours must be between 1 and 24")

        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = await self.async_client.get(
            f"/intensity/stats/{from_str}/{to_str}/{block_hours}"
        )
        data = self._handle_response(response)
        return StatisticsResponse.model_validate(data).data

    # Generation mix endpoints
    def get_current_generation_mix(self) -> GenerationData:
        """Get generation mix for current half hour."""
        response = self._client.get("/generation")
        data = self._handle_response(response)
        return GenerationResponse.model_validate(data).data

    async def aget_current_generation_mix(self) -> GenerationData:
        """Get generation mix for current half hour (async)."""
        response = await self.async_client.get("/generation")
        data = self._handle_response(response)
        return GenerationResponse.model_validate(data).data

    def get_generation_mix_past_24h(
        self, from_dt: Union[datetime, str]
    ) -> GenerationData:
        """Get generation mix for past 24 hours from datetime."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(f"/generation/{from_str}/pt24h")
        data = self._handle_response(response)
        return GenerationResponse.model_validate(data).data

    async def aget_generation_mix_past_24h(
        self, from_dt: Union[datetime, str]
    ) -> GenerationData:
        """Get generation mix for past 24 hours from datetime (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(f"/generation/{from_str}/pt24h")
        data = self._handle_response(response)
        return GenerationResponse.model_validate(data).data

    def get_generation_mix_between_datetimes(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str]
    ) -> GenerationData:
        """Get generation mix between two datetimes."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = self._client.get(f"/generation/{from_str}/{to_str}")
        data = self._handle_response(response)
        return GenerationResponse.model_validate(data).data

    async def aget_generation_mix_between_datetimes(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str]
    ) -> GenerationData:
        """Get generation mix between two datetimes (async)."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = await self.async_client.get(f"/generation/{from_str}/{to_str}")
        data = self._handle_response(response)
        return GenerationResponse.model_validate(data).data

    # Regional endpoints
    def get_current_regional_intensity(self) -> List[RegionalFromTo]:
        """Get regional carbon intensity data for current half hour."""
        response = self._client.get("/regional")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    async def aget_current_regional_intensity(self) -> List[RegionalFromTo]:
        """Get regional carbon intensity data for current half hour (async)."""
        response = await self.async_client.get("/regional")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    def get_current_england_intensity(self) -> List[RegionalFromTo]:
        """Get carbon intensity data for England."""
        response = self._client.get("/regional/england")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    async def aget_current_england_intensity(self) -> List[RegionalFromTo]:
        """Get carbon intensity data for England (async)."""
        response = await self.async_client.get("/regional/england")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    def get_current_scotland_intensity(self) -> List[RegionalFromTo]:
        """Get carbon intensity data for Scotland."""
        response = self._client.get("/regional/scotland")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    async def aget_current_scotland_intensity(self) -> List[RegionalFromTo]:
        """Get carbon intensity data for Scotland (async)."""
        response = await self.async_client.get("/regional/scotland")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    def get_current_wales_intensity(self) -> List[RegionalFromTo]:
        """Get carbon intensity data for Wales."""
        response = self._client.get("/regional/wales")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    async def aget_current_wales_intensity(self) -> List[RegionalFromTo]:
        """Get carbon intensity data for Wales (async)."""
        response = await self.async_client.get("/regional/wales")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    def get_intensity_by_postcode(self, postcode: str) -> List[RegionalId]:
        """Get carbon intensity data for specific postcode."""
        response = self._client.get(f"/regional/postcode/{postcode}")
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_intensity_by_postcode(self, postcode: str) -> List[RegionalId]:
        """Get carbon intensity data for specific postcode (async)."""
        response = await self.async_client.get(f"/regional/postcode/{postcode}")
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    def get_intensity_by_region_id(self, region_id: int) -> List[RegionalId]:
        """Get carbon intensity data for specific region ID (1-17)."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        response = self._client.get(f"/regional/regionid/{region_id}")
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_intensity_by_region_id(self, region_id: int) -> List[RegionalId]:
        """Get carbon intensity data for specific region ID (async)."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        response = await self.async_client.get(f"/regional/regionid/{region_id}")
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    # Regional forward/past 24h/48h endpoints
    def get_regional_intensity_forward_24h(
        self, from_dt: Union[datetime, str]
    ) -> List[RegionalFromTo]:
        """Get regional carbon intensity data 24 hours forward."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(f"/regional/intensity/{from_str}/fw24h")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    async def aget_regional_intensity_forward_24h(
        self, from_dt: Union[datetime, str]
    ) -> List[RegionalFromTo]:
        """Get regional carbon intensity data 24 hours forward (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(f"/regional/intensity/{from_str}/fw24h")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    def get_regional_intensity_forward_24h_postcode(
        self, from_dt: Union[datetime, str], postcode: str
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 24 hours forward for postcode."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(
            f"/regional/intensity/{from_str}/fw24h/postcode/{postcode}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_regional_intensity_forward_24h_postcode(
        self, from_dt: Union[datetime, str], postcode: str
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 24 hours forward for postcode (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(
            f"/regional/intensity/{from_str}/fw24h/postcode/{postcode}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    def get_regional_intensity_forward_24h_region(
        self, from_dt: Union[datetime, str], region_id: int
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 24 hours forward for region."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        from_str = self._format_datetime(from_dt)
        response = self._client.get(
            f"/regional/intensity/{from_str}/fw24h/regionid/{region_id}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_regional_intensity_forward_24h_region(
        self, from_dt: Union[datetime, str], region_id: int
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 24 hours forward for region (async)."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(
            f"/regional/intensity/{from_str}/fw24h/regionid/{region_id}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    def get_regional_intensity_forward_48h(
        self, from_dt: Union[datetime, str]
    ) -> List[RegionalFromTo]:
        """Get regional carbon intensity data 48 hours forward."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(f"/regional/intensity/{from_str}/fw48h")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    async def aget_regional_intensity_forward_48h(
        self, from_dt: Union[datetime, str]
    ) -> List[RegionalFromTo]:
        """Get regional carbon intensity data 48 hours forward (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(f"/regional/intensity/{from_str}/fw48h")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    def get_regional_intensity_forward_48h_postcode(
        self, from_dt: Union[datetime, str], postcode: str
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 48 hours forward for postcode."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(
            f"/regional/intensity/{from_str}/fw48h/postcode/{postcode}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_regional_intensity_forward_48h_postcode(
        self, from_dt: Union[datetime, str], postcode: str
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 48 hours forward for postcode (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(
            f"/regional/intensity/{from_str}/fw48h/postcode/{postcode}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    def get_regional_intensity_forward_48h_region(
        self, from_dt: Union[datetime, str], region_id: int
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 48 hours forward for region."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        from_str = self._format_datetime(from_dt)
        response = self._client.get(
            f"/regional/intensity/{from_str}/fw48h/regionid/{region_id}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_regional_intensity_forward_48h_region(
        self, from_dt: Union[datetime, str], region_id: int
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 48 hours forward for region (async)."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(
            f"/regional/intensity/{from_str}/fw48h/regionid/{region_id}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    def get_regional_intensity_past_24h(
        self, from_dt: Union[datetime, str]
    ) -> List[RegionalFromTo]:
        """Get regional carbon intensity data 24 hours in the past."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(f"/regional/intensity/{from_str}/pt24h")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    async def aget_regional_intensity_past_24h(
        self, from_dt: Union[datetime, str]
    ) -> List[RegionalFromTo]:
        """Get regional carbon intensity data 24 hours in the past (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(f"/regional/intensity/{from_str}/pt24h")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    def get_regional_intensity_past_24h_postcode(
        self, from_dt: Union[datetime, str], postcode: str
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 24 hours in the past for postcode."""
        from_str = self._format_datetime(from_dt)
        response = self._client.get(
            f"/regional/intensity/{from_str}/pt24h/postcode/{postcode}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_regional_intensity_past_24h_postcode(
        self, from_dt: Union[datetime, str], postcode: str
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 24 hours in the past for postcode (async)."""
        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(
            f"/regional/intensity/{from_str}/pt24h/postcode/{postcode}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    def get_regional_intensity_past_24h_region(
        self, from_dt: Union[datetime, str], region_id: int
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 24 hours in the past for region."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        from_str = self._format_datetime(from_dt)
        response = self._client.get(
            f"/regional/intensity/{from_str}/pt24h/regionid/{region_id}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_regional_intensity_past_24h_region(
        self, from_dt: Union[datetime, str], region_id: int
    ) -> List[RegionalId]:
        """Get regional carbon intensity data 24 hours in the past for region (async)."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        from_str = self._format_datetime(from_dt)
        response = await self.async_client.get(
            f"/regional/intensity/{from_str}/pt24h/regionid/{region_id}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    def get_regional_intensity_between_datetimes(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str]
    ) -> List[RegionalFromTo]:
        """Get regional carbon intensity data between datetimes."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = self._client.get(f"/regional/intensity/{from_str}/{to_str}")
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    async def aget_regional_intensity_between_datetimes(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str]
    ) -> List[RegionalFromTo]:
        """Get regional carbon intensity data between datetimes (async)."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = await self.async_client.get(
            f"/regional/intensity/{from_str}/{to_str}"
        )
        data = self._handle_response(response)
        return RegionalFromToResponse.model_validate(data).data

    def get_regional_intensity_between_datetimes_postcode(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str], postcode: str
    ) -> List[RegionalId]:
        """Get regional carbon intensity data between datetimes for postcode."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = self._client.get(
            f"/regional/intensity/{from_str}/{to_str}/postcode/{postcode}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_regional_intensity_between_datetimes_postcode(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str], postcode: str
    ) -> List[RegionalId]:
        """Get regional carbon intensity data between datetimes for postcode (async)."""
        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = await self.async_client.get(
            f"/regional/intensity/{from_str}/{to_str}/postcode/{postcode}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    def get_regional_intensity_between_datetimes_region(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str], region_id: int
    ) -> List[RegionalId]:
        """Get regional carbon intensity data between datetimes for region."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = self._client.get(
            f"/regional/intensity/{from_str}/{to_str}/regionid/{region_id}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data

    async def aget_regional_intensity_between_datetimes_region(
        self, from_dt: Union[datetime, str], to_dt: Union[datetime, str], region_id: int
    ) -> List[RegionalId]:
        """Get regional carbon intensity data between datetimes for region (async)."""
        if not 1 <= region_id <= 17:
            raise ValueError(_REGION_ID_ERROR)

        from_str = self._format_datetime(from_dt)
        to_str = self._format_datetime(to_dt)
        response = await self.async_client.get(
            f"/regional/intensity/{from_str}/{to_str}/regionid/{region_id}"
        )
        data = self._handle_response(response)
        return RegionalIdResponse.model_validate(data).data



"""Constants and utility mappings for the UK Carbon Intensity API."""

from typing import Dict

# Region ID to name mapping
REGION_NAMES: Dict[int, str] = {
    1: "North Scotland",
    2: "South Scotland",
    3: "North West England",
    4: "North East England",
    5: "Yorkshire",
    6: "North Wales",
    7: "South Wales",
    8: "West Midlands",
    9: "East Midlands",
    10: "East England",
    11: "South West England",
    12: "South England",
    13: "London",
    14: "South East England",
    15: "England",
    16: "Scotland",
    17: "Wales",
}

# Reverse mapping: name to region ID
REGION_IDS: Dict[str, int] = {name.lower(): id_ for id_, name in REGION_NAMES.items()}

# Fuel type descriptions
FUEL_DESCRIPTIONS = {
    "gas": "Natural Gas",
    "coal": "Coal",
    "biomass": "Biomass",
    "nuclear": "Nuclear",
    "hydro": "Hydroelectric",
    "imports": "Electricity Imports",
    "other": "Other Sources",
    "wind": "Wind Power",
    "solar": "Solar Power",
    "storage": "Pumped Storage",
}

# API endpoints
BASE_URL = "https://api.carbonintensity.org.uk"

# Default timeout for HTTP requests
DEFAULT_TIMEOUT = 30.0

# User agent for requests
USER_AGENT_PREFIX = "uk-grid-intensity-python-client"



"""Simple test to verify the package works."""

import sys
import os

# Add the src directory to Python path for testing
sys.path.insert(0, os.path.join(os.path.dirname(__file__), "src"))

from uk_grid_intensity import CarbonIntensityClient, CarbonIntensityAPIError


def test_import():
    """Test that imports work correctly."""
    print("✓ Import test passed")


def test_client_creation():
    """Test that we can create a client."""
    client = CarbonIntensityClient()
    assert client is not None
    print("✓ Client creation test passed")


def test_exception():
    """Test that we can create the exception."""
    exc = CarbonIntensityAPIError("Test error", 400)
    assert exc.message == "Test error"
    assert exc.status_code == 400
    print("✓ Exception test passed")


if __name__ == "__main__":
    print("Running basic tests...")

    try:
        test_import()
        test_client_creation()
        test_exception()
        print("\n✅ All tests passed!")
    except Exception as e:
        print(f"\n❌ Test failed: {e}")
        sys.exit(1)



"""Command-line interface for the UK Grid Carbon Intensity API client."""

import argparse
import sys
from datetime import datetime
from typing import Optional

from .client import CarbonIntensityClient, CarbonIntensityAPIError
from ._version import __version__


def format_intensity_data(data_list):
    """Format intensity data for display."""
    if not data_list:
        return "No data available."

    output = []
    for item in data_list:
        output.append(f"Period: {item.from_} to {item.to}")
        output.append(f"  Forecast: {item.intensity.forecast} gCO2/kWh")
        if item.intensity.actual:
            output.append(f"  Actual: {item.intensity.actual} gCO2/kWh")
        output.append(f"  Index: {item.intensity.index}")
        output.append("")
    return "\n".join(output)


def format_generation_data(data):
    """Format generation mix data for display."""
    if not data:
        return "No data available."

    output = []
    output.append(f"Period: {data.from_} to {data.to}")
    output.append("  Generation Mix:")
    for fuel in data.generationmix:
        output.append(f"    {fuel.fuel}: {fuel.perc}%")
    return "\n".join(output)


def main():
    """Main CLI entry point."""
    parser = argparse.ArgumentParser(
        description="UK Grid Carbon Intensity API Client",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )

    parser.add_argument(
        "--version", action="version", version=f"uk-grid-intensity {__version__}"
    )

    subparsers = parser.add_subparsers(dest="command", help="Available commands")

    # Current intensity command
    current_parser = subparsers.add_parser(
        "current", help="Get current carbon intensity"
    )
    current_parser.add_argument(
        "--format", choices=["text", "json"], default="text", help="Output format"
    )

    # Intensity by date command
    date_parser = subparsers.add_parser("date", help="Get intensity for specific date")
    date_parser.add_argument("date", help="Date in YYYY-MM-DD format")
    date_parser.add_argument(
        "--format", choices=["text", "json"], default="text", help="Output format"
    )

    # Generation mix command
    generation_parser = subparsers.add_parser(
        "generation", help="Get current generation mix"
    )
    generation_parser.add_argument(
        "--format", choices=["text", "json"], default="text", help="Output format"
    )

    # Regional command
    regional_parser = subparsers.add_parser("regional", help="Get regional data")
    regional_group = regional_parser.add_mutually_exclusive_group()
    regional_group.add_argument("--postcode", help="Postcode (e.g., SW1, RG41)")
    regional_group.add_argument("--region-id", type=int, help="Region ID (1-17)")
    regional_group.add_argument(
        "--country", choices=["england", "scotland", "wales"], help="Country name"
    )
    regional_parser.add_argument(
        "--format", choices=["text", "json"], default="text", help="Output format"
    )

    # Factors command
    factors_parser = subparsers.add_parser(
        "factors", help="Get carbon intensity factors"
    )
    factors_parser.add_argument(
        "--format", choices=["text", "json"], default="text", help="Output format"
    )

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
        return 1

    try:
        client = CarbonIntensityClient()

        if args.command == "current":
            data = client.get_current_intensity()
            if args.format == "json":
                import json

                print(
                    json.dumps(
                        [item.model_dump() for item in data], indent=2, default=str
                    )
                )
            else:
                print("Current Carbon Intensity:")
                print(format_intensity_data(data))

        elif args.command == "date":
            data = client.get_intensity_by_date(args.date)
            if args.format == "json":
                import json

                print(
                    json.dumps(
                        [item.model_dump() for item in data], indent=2, default=str
                    )
                )
            else:
                print(f"Carbon Intensity for {args.date}:")
                print(format_intensity_data(data))

        elif args.command == "generation":
            data = client.get_current_generation_mix()
            if args.format == "json":
                import json

                print(
                    json.dumps(
                        [item.model_dump() for item in data], indent=2, default=str
                    )
                )
            else:
                print("Current Generation Mix:")
                print(format_generation_data(data))

        elif args.command == "regional":
            if args.postcode:
                data = client.get_intensity_by_postcode(args.postcode)
            elif args.region_id:
                data = client.get_intensity_by_region_id(args.region_id)
            elif args.country == "england":
                data = client.get_current_england_intensity()
            elif args.country == "scotland":
                data = client.get_current_scotland_intensity()
            elif args.country == "wales":
                data = client.get_current_wales_intensity()
            else:
                data = client.get_current_regional_intensity()

            if args.format == "json":
                import json

                print(
                    json.dumps(
                        [item.model_dump() for item in data], indent=2, default=str
                    )
                )
            else:
                print("Regional Carbon Intensity:")
                # Regional data formatting would need more complex logic
                print("Use --format json for detailed output")

        elif args.command == "factors":
            data = client.get_intensity_factors()
            if args.format == "json":
                import json

                print(
                    json.dumps(
                        [item.model_dump() for item in data], indent=2, default=str
                    )
                )
            else:
                print("Carbon Intensity Factors:")
                for item in data:
                    for field_name, field_value in item.model_dump().items():
                        if field_value is not None:
                            print(f"  {field_name}: {field_value} gCO2/kWh")

    except CarbonIntensityAPIError as e:
        print(f"API Error: {e.message}", file=sys.stderr)
        if e.status_code:
            print(f"Status Code: {e.status_code}", file=sys.stderr)
        return 1
    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        return 1

    return 0


if __name__ == "__main__":
    sys.exit(main())
